@@ -1,18 +1,13 @@
-deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hemqle","sch"),nboot=500
-         ){ 
- if(any(method %in% c("l","b","ml","hosc","hesc","hemqle"))){            
+deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hemqle","wh"),
+       kwh=NULL, nboot=500){ 
+ if(!any(method %in% c("l","d","ml","hosc","hesc","hemqle","wh")))
+ 	stop("not a valid method.")          
      if(is.null(catch)) 
-         stop ("catch vector does not exist")
+         stop ("catch vector does not exist.")
      if(is.null(effort)) 
-         stop ("effort vector does not exist")
+         stop ("effort vector does not exist.")
       if(length(catch)!=length(effort)) 
-        stop("unequal vector lengths")
-   }
-
-  if(any(method=="sch")){
-     if(is.null(catch)) 
-         stop ("catch vector does not exist") 
-     }
+        stop("unequal catch/effort vector lengths.")
     x<-as.data.frame(cbind(catch,effort))
     names(x)<-c("catch","effort")
     x$samp<-seq(1,length(x$catch),1)
@@ -21,13 +16,17 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
     nsam<-length(x$samp)
     x$K[c(seq(2,nsam,1))]<-cumsum(x$catch[c(seq(1,nsam-1,1))])
     x$F[c(seq(2,nsam,1))]<-cumsum(x$effort[c(seq(1,nsam-1,1))])
-
-  if(length(x$catch)<3) stop("only two observations")
-         
+   
+  if(length(x$catch)<2) stop(">=2 observations required!")
+  if(length(x$catch)==2){
+       if(x$catch[2]>=x$catch[1]) warning ("For the two-pass estimator, catch of first pass must be greater than catch of second pass.")
+     }
+       
 #estimate N and q using Leslie and Davis method
-  if(length(x$catch)>=3){
-     if(any(method=="l")){
+      if(any(method=="l")){
+       if(length(x$catch)<3) stop ("Leslie method requires at least 3 observations!")
        ld<-lm(cpue~K,data=x)
+        l.out<-NULL
        names(ld$coefficients)[2]<-"q"
        Nld<--coef(summary(ld))[1]/coef(summary(ld))[2]
        qld<--coef(summary(ld))[2]
@@ -48,10 +47,15 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
          ans$anova<-anova(ld)   
          ans$summary<-summary(ld) 
          ans$residuals<-as.vector(residuals(ld))
+   
          l.out<<-ans
      }
 #estimate N and q using effort-corrected Delury method
     if(any(method=="d")){
+      if(length(x$catch)<3) stop ("Delury method requires at least 3 observations!")
+      if(any(x$catch==0))warning("Zero catches not allowed in Delury method.")
+      d.out<-NULL
+      if(!any(x$catch==0)){ 
   	x$E<-x$F+x$effort*0.5
    	cd<-lm(log(cpue)~E,data=x)
       names(cd$coefficients)[2]<-"q"
@@ -76,114 +80,21 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
          ans$summary<-summary(cd) 
          ans$residuals<-as.vector(residuals(cd))
          d.out<<-ans
-     }
-
- if(any(method=="sch")){
-     	nsam<-length(catch)
-	pcatch1<-c(rep(0,nsam))
-	T<-c(rep(0,nsam))
-	T[c(seq(1,nsam,1))]<-cumsum(catch[c(seq(1,nsam,1))])
-	TC<-sum(catch)
-	N<-TC*1.2
-	q1<-T[nsam]/(nsam*N-sum(T[1:nsam-1]))
-	parms1<-c(N,q1)                                   
-      model1<-function(x){					
-   		N<-x[1]
-  		q1<-x[2]
-  		pcatch1[seq(1,nsam,1)]<-N*q1*(1-q1)^(seq(1,nsam,1)-1)
-   		pT<-sum(pcatch1)
- 		L1<-sum(log((seq(1,T[nsam],1)+N-T[nsam])/seq(1,T[nsam],1)))
-  		H<-sum(catch[seq(1,nsam,1)]*log(catch[seq(1,nsam,1)]/pcatch1[seq(1,nsam,1)]))  
-  		G<-N*log(N)-T[nsam]*log(T[nsam])-(N-T[nsam])*log(N-pT)-L1
-   		G+H
-	   }   
-	lower<-c(sum(catch),0)                  
-	upper<-c(sum(catch)*20,2)                 #upper limits of N and q
-	results1<-optim(parms1, model1, gr = NULL,lower=lower,upper=upper,method=c("L-BFGS-B"), 
-      control=list(maxit=100000),hessian=TRUE)
-	cov1<-solve(results1$hessian)
-      pcatch1[seq(1,nsam,1)]<-results1$par[1]*results1$par[2]*(1-results1$par[2])^(seq(1,nsam,1)-1)
-      resid1<-catch-pcatch1
-         t<-1.96
-         ans<-NULL  
-         ans$Model_1<-matrix(NA,2L,4L)
-         ans$Model_1<-rbind(cbind(round(results1$par[1],1),
-                              round(sqrt(cov1[1,1]),2),
-                              round(results1$par[1],1)-t*round(sqrt(cov1[1,1]),2),
-                              round(results1$par[1],1)+t*round(sqrt(cov1[1,1]),2)),
-                            cbind(results1$par[2],sqrt(cov1[2,2]),
-                               results1$par[2]-t*sqrt(cov1[2,2]),
-                               results1$par[2]+t*sqrt(cov1[2,2])))
-         dimnames(ans$Model_1)<-list(cbind("N","q1"),c("Estimate","SE","95% LCI","95% UCI"))
-         ans$Model1_Parameter_Correlation<-cov2cor(cov1)
-      #Model 2  
-	pcatch2<-c(rep(0,nsam))
-      T<-c(rep(0,nsam))
-	T[c(seq(1,nsam,1))]<-cumsum(catch[c(seq(1,nsam,1))])
-	TC<-sum(catch)
-	N<-TC*1.2
-	q12<-catch[1]/N
-	q2<-(T[nsam]-catch[1])/
-      	((nsam-1)*(N-catch[1])-sum(T[1:nsam-1]-catch[1]))
-  	parms2<-c(N,q12,q2)
-  	model2<-function(x){	
-   		N<-x[1]
-  		q12<-x[2]
-   		q2<-x[3]
-   		pcatch2[1]<-q12*N
-   		pcatch2[seq(2,nsam,1)]<-N*q2*(1-q12)*(1-q2)^(seq(2,nsam,1)-2)
-   		pT<-sum(pcatch2)
-  		L1<-sum(log((seq(1,T[nsam],1)+N-T[nsam])/seq(1,T[nsam],1)))
-  		H<-sum(catch[seq(1,nsam,1)]*log(catch[seq(1,nsam,1)]/pcatch2[seq(1,nsam,1)]))  
-  		G<-N*log(N)-T[nsam]*log(T[nsam])-(N-T[nsam])*log(N-pT)-L1
-   		G+H
-		}
-	lower<-c(sum(catch),0,0)                   #lower limits of N and q
-	upper<-c(sum(catch)*20,2,2)                 #upper limits of N and q
-	results2<-optim(parms2, model2, gr = NULL,lower=lower,upper=upper,method=c("L-BFGS-B"), 
-      control=list(maxit=100000),hessian=TRUE)
-	cov2<-solve(results2$hessian)
-      	pcatch2[1]<-results2$par[2]*results2$par[1]
-   		pcatch2[seq(2,nsam,1)]<-results2$par[1]*results2$par[3]*(1-results2$par[2])*(1-results2$par[3])^(seq(2,nsam,1)-2)
-   		resid2<-catch-pcatch2
-
-         t<-1.96
-         ans$Model_2<-matrix(NA,3L,4L)
-         ans$Model_2<-rbind(cbind(round(results2$par[1],1),round(sqrt(cov2[1,1]),2),
-                              round(results2$par[1],1)-t*round(sqrt(cov2[1,1]),2),
-                              round(results2$par[1],1)+t*round(sqrt(cov2[1,1]),2)),
-                            cbind(results2$par[2],sqrt(cov2[2,2]),
-                              results2$par[2]-t*sqrt(cov2[2,2]),
-                              results2$par[2]+t*sqrt(cov2[2,2])),
-                            cbind(results2$par[3],sqrt(cov2[3,3]),
-                              results2$par[3]-t*sqrt(cov2[3,3]),
-                              results2$par[3]+t*sqrt(cov2[3,3])))
-         dimnames(ans$Model_2)<-list(cbind("N","q1","q2"),c("Estimate","SE","95% LCI","95% UCI"))
-          ans$Model2_Parameter_Correlation<-cov2cor(cov2)
-
-         ans$gof<-matrix(NA,2L,3L)
-         ans$gof<-rbind(cbind(sum((catch-pcatch1)^2/pcatch1),nsam-2),
-                        cbind(sum((catch-pcatch2)^2/pcatch2),nsam-3)) 
-                   
-         dimnames(ans$gof)<-list(c("Model 1","Model 2"),c("Chi-square","df"))
-
-	   ans$comparison<-matrix(NA,3L,1L)
-         ans$comparison<-rbind(round(results1$value,2),round(results2$value,2),
-                          2*(round(results1$value,2)-round(results2$value,2)))
-          dimnames(ans$comparison)<-list(c("Model 1 F","Model 2 F","Chi-square"),c("Value"))
-          ans$residuals<-data.frame(Model_1=resid1,Model_2=resid2)
-          sch.out<<-ans
-  }
-    
+     } #if
+     } #if
+ 
      if(any(method=="ml")){
+       if(length(x$catch)<3) stop ("ml method requires at least 3 observations!")
+        ml.out<-NULL 
         dd<-x
+        Q<-NULL
         parms<-c(k=0.0005)
         ml<-function(y){
           k<-y[1] 
           dd$p<-1-exp(-k*dd$effort)
           dd$q<-1-dd$p
             for(i in 1:as.numeric(length(dd$catch))){
-              dd$nf[i]<-sum(log(seq(1,dd$catch[i],1)))
+             dd$nf[i]<-lfactorial(dd$catch[i])
               if(i==1) dd$qp[i]<-dd$p[i]
               if(i>1)  dd$qp[i]<-dd$p[i]*prod(dd$q[1:i-1])
             }
@@ -199,14 +110,15 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
        Korig<-Kout$minimum
        L<-Kout$objective
        Norig<-sum(dd$catch)/(1-Q)
-       
+        Qb<-NULL
         Bout<-data.frame(k=NULL,N=NULL)
         mlb<-function(y){
           k<-y[1]  
+         
           Bdata$p<-1-exp(-k*Bdata$effort)
           Bdata$q<-1-Bdata$p
             for(i in 1:as.numeric(length(Bdata$catch))){
-              dd$nf[i]<-sum(log(seq(1,Bdata$catch[i],1)))
+              Bdata$nf[i]<-lfactorial(Bdata$catch[i])
               if(i==1) Bdata$qp[i]<-Bdata$p[i]
               if(i>1)  Bdata$qp[i]<-Bdata$p[i]*prod(Bdata$q[1:i-1])
             }
@@ -219,7 +131,6 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
          }
        for(i in 1:nboot){
        newc<-rmultinom(1, Norig, c(dd$catch/Norig,1-(sum(dd$catch)/Norig)))
-       newc<-ifelse(newc==0,1,newc)
        Bdata<-data.frame(catch=newc[-length(newc)],effort=dd$effort)
        upper<--log(0.0005)/max(Bdata$effort)
        Kb<-optimize(mlb,lower=0,upper=upper,tol=0.0000000001)$minimum
@@ -236,7 +147,7 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
          ans$results<-matrix(NA,2L,6L)
          ans$results<-rbind(cbind(round(Norig,0),round(seN,1),round(LCn,1),round(UCn,1)),
                   cbind(Korig,sek,LCq,UCq))
-         dimnames(ans$results)<-list(cbind("N","q"),c("Estimate","SE","95% LCI","95% UCI"))
+         dimnames(ans$results)<-list(cbind("N","k"),c("Estimate","SE","95% LCI","95% UCI"))
           dd$p<-1-exp(-Korig*dd$effort)
           dd$q<-1-dd$p
             for(i in 1:as.numeric(length(dd$catch))){
@@ -251,12 +162,22 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
          dimnames(ans$gof)<-list(c("Chi-square","df"),"Value")
          ans$residuals<-as.vector(dd$catch-dd$pred)
          ml.out<<-ans
+     
     }  
  
 #Homogeneous Sample Coverage
  if(any(method=="hosc")){
-     dd<-x
-      nsam<-length(x[,1])
+       hosc.out<-NULL
+       dd<-x
+       dd$del<-ifelse(dd$catch>0,1,0)
+       ld<-length(dd$catch)       
+       for(i in 1:as.numeric(ld-1)){
+          dd$del[i]<-ifelse(dd$catch[i]==0 & 
+             any(dd$catch[i:as.numeric(ld)]>0),1,dd$del[i])
+          }
+       dd<-dd[dd$del==1,]
+       kk<-dd
+      nsam<-length(dd[,1])
       dd$Dk<-cumsum(dd$catch)
       dd$ue<-dd$catch/dd$effort
       for(i in 1:as.numeric(nsam-1)){dd$Cp[i]<-1-(dd$ue[i+1]/(dd$ue[1]))} 
@@ -265,12 +186,11 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
                dd$check))
         tau<-ifelse(any(dd$check==TRUE),which(dd$check==1)-1,nsam-1)
         Nsc0<-dd$Dk[tau]/dd$Cp[tau]
+        q<-dd$catch[1]/(Nsc0*dd$effort[1])
         prob<-c(dd$catch/Nsc0,1-(sum(dd$catch)/Nsc0))
-        BootNsc0<-data.frame(rep=NULL,Nscb=NULL)
-
+        BootNsc0<-data.frame(rep=NULL,Nscb=NULL,qscb=NULL)
 
      #Bootstrap
-        kk<-x
      for(j in 1:nboot){
        kk$catch<-rmultinom(1,Nsc0,prob)[1:nsam]
        kk$Dk<-cumsum(kk$catch)
@@ -281,24 +201,38 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
                kk$check))
        tauk<-ifelse(any(kk$check==TRUE),which(kk$check==1)-1,nsam-1)
        Nscb<-kk$Dk[tauk]/kk$Cp[tauk]
-       BootNsc0[j,1]<-1
+       qscb<-kk$catch[1]/(Nscb*kk$effort[1])
+       BootNsc0[j,1]<-j
        BootNsc0[j,2]<-Nscb
+       BootNsc0[j,3]<-qscb
+
       }
          seN<-sd(BootNsc0[,2])
+         seqq<-sd(BootNsc0[,3])
          C<-exp(1.96*sqrt(log(1+((seN^2)/((Nsc0-sum(dd$catch))^2)))))
          LCn<-sum(dd$catch)+(Nsc0-sum(dd$catch))/C
          UCn<-sum(dd$catch)+(Nsc0-sum(dd$catch))*C
        
          ans<-NULL  
-         ans$results<-matrix(NA,1L,6L)
-         ans$results<-rbind(cbind(round(Nsc0,0),round(seN,1),round(LCn,1),round(UCn,1)))   
-         dimnames(ans$results)<-list(cbind("N"),c("Estimate","SE","95% LCI","95% UCI"))  
-         hosc.out<<-ans
+         ans$results<-matrix(NA,2L,6L)
+         ans$results<-rbind(cbind(round(Nsc0,0),round(seN,1),round(LCn,1),round(UCn,1)),   
+                        cbind(q,seqq,NA,NA))    
+         dimnames(ans$results)<-list(cbind("N","lambda"),c("Estimate","SE","95% LCI","95% UCI"))          
+        hosc.out<<-ans
    }  
 
  #Heterogenous Sample Coverage
    if(any(method=="hesc")){
+      hesc.out<-NULL
       dd<-x
+       dd$del<-ifelse(dd$catch>0,1,0)
+       ld<-length(dd$catch)       
+       for(i in 1:as.numeric(ld-1)){
+          dd$del[i]<-ifelse(dd$catch[i]==0 & 
+             any(dd$catch[i:as.numeric(ld)]>0),1,dd$del[i])
+          }
+       dd<-dd[dd$del==1,]
+        kk<-dd
       nsam<-length(dd[,1])
       dd$Dk<-cumsum(dd$catch)
       dd$ue<-dd$catch/dd$effort
@@ -311,12 +245,13 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
      epsilon<-max(Nsc0*((dd$catch[1]-dd$catch[2]*(dd$effort[1]/dd$effort[2]))/(dd$catch[1]^2))-1,0)
      dd$Ak<-cumsum(dd$effort)/dd$effort
      Nsc1<-(dd$Dk[tau]/dd$Cp[tau])+(dd$Ak[tau]*dd$catch[tau]*epsilon)/dd$Cp[tau]
+     q<-dd$catch[1]/(Nsc1*dd$effort[1])
      probb<-c(dd$catch/Nsc1,1-(sum(dd$catch)/Nsc1))
-     BootNsc1<-data.frame(rep=NULL,Nscb=NULL)
+     BootNsc1<-data.frame(rep=NULL,Nscb=NULL,qscb=NULL)
      CVNsc1<-sqrt(epsilon)
 
      #Bootstrap
-        kk<-x
+       
      for(j in 1:nboot){
        kk$catch<-rmultinom(1,Nsc1,probb)[1:nsam]
        kk$Dk<-cumsum(kk$catch)
@@ -330,23 +265,36 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
        epsilonb<-max(Nscb0*((kk$catch[1]-kk$catch[2]*(kk$effort[1]/kk$effort[2]))/(kk$catch[1]^2))-1,0)
        kk$Akb<-cumsum(kk$effort)/kk$effort
        Nsc1b<-(kk$Dk[tauk]/kk$Cp[tauk])+(kk$Akb[tauk]*kk$catch[tauk]*epsilonb)/kk$Cp[tauk]
-       BootNsc1[j,1]<-1
+       qsc1b<-kk$catch[1]/(Nsc1b*kk$effort[1])
+       BootNsc1[j,1]<-j
        BootNsc1[j,2]<-Nsc1b
+       BootNsc1[j,3]<-qsc1b
       }
-         seN<-sd(BootNsc1[,2])
+         seN<-sd(BootNsc1[,2]) 
+         seqq<-sd(BootNsc1[,3]) 
          C<-exp(1.96*sqrt(log(1+((seN^2)/((Nsc1-sum(dd$catch))^2)))))
          LCn<-sum(dd$catch)+(Nsc1-sum(dd$catch))/C
          UCn<-sum(dd$catch)+(Nsc1-sum(dd$catch))*C
        
          ans<-NULL  
-         ans$results<-matrix(NA,1L,7L)
-         ans$results<-rbind(cbind(round(Nsc1,0),round(seN,1),round(LCn,1),round(UCn,1),round(CVNsc1,3)))   
-         dimnames(ans$results)<-list(cbind("N"),c("Estimate","SE","95% LCI","95% UCI"," lambda CV"))  
+         ans$results<-matrix(NA,2L,7L)
+         ans$results<-rbind(cbind(round(Nsc1,0),round(seN,1),round(LCn,1),round(UCn,1),round(CVNsc1,3)),
+                             cbind(q,seqq,NA,NA,NA))    
+         dimnames(ans$results)<-list(cbind("N","lambda"),c("Estimate","SE","95% LCI","95% UCI"," lambda CV"))  
          hesc.out<<-ans
    }
   #Heterogenous MQLE
    if(any(method=="hemqle")){
+     hemqle.out<-NULL
      dd<-x
+       dd$del<-ifelse(dd$catch>0,1,0)
+       ld<-length(dd$catch)       
+       for(i in 1:as.numeric(ld-1)){
+          dd$del[i]<-ifelse(dd$catch[i]==0 & 
+             any(dd$catch[i:as.numeric(ld)]>0),1,dd$del[i])
+          }
+       dd<-dd[dd$del==1,]
+       kk<-dd
       nsam<-length(dd[,1])
       dd$Dk<-cumsum(dd$catch)
       dd$ue<-dd$catch/dd$effort
@@ -360,22 +308,24 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
        dd$Ak<-cumsum(dd$effort)/dd$effort
        dd$I<-ifelse(dd$Cp>0,1,0)
      for(i in 1:nsam){
-        if(i==1){ dd$sql[i]<-0;dd$sql2<-0}
-        if(i>1){
-           dd$sql[i]<-(dd$effort[i]^2/dd$catch[i])*(dd$Dk[i-1]+(dd$Ak[i-1]*dd$catch[i-1]*epsilon))*dd$I[i-1]
-           dd$sql2[i]<-(dd$effort[i]^2/dd$catch[i])*dd$Cp[i-1]*dd$I[i-1]
-          }
+         if(dd$catch[i]>0){
+           if(i==1){dd$sql[i]<-0;dd$sql2<-0}
+           if(i>1){
+             dd$sql[i]<-(dd$effort[i]^2/dd$catch[i])*(dd$Dk[i-1]+(dd$Ak[i-1]*dd$catch[i-1]*epsilon))*dd$I[i-1]
+             dd$sql2[i]<-(dd$effort[i]^2/dd$catch[i])*dd$Cp[i-1]*dd$I[i-1]
+            }
+         }
+        if(dd$catch[i]==0){dd$sql[i]<-0;dd$sql2[i]<-0}  
      }
      Nmqle<-sum(dd$sql)/sum(dd$sql2)
+     q<-dd$catch[1]/(Nmqle*dd$effort[1])
      probb<-c(dd$catch/Nmqle,1-(sum(dd$catch)/Nmqle))
-     BootNmq<-data.frame(rep=NULL,Nscb=NULL)
+     BootNmq<-data.frame(rep=NULL,Nscb=NULL,qscb=NULL)
      CVNmqle<-sqrt(epsilon)
 
      #Bootstrap
-        kk<-x
      for(j in 1:nboot){     
        kk$catch<-rmultinom(1,Nmqle,probb)[1:nsam]
-       kk$catch<-ifelse(kk$catch==0,1,kk$catch)
        kk$Dk<-cumsum(kk$catch)
        kk$ue<-kk$catch/kk$effort
        for(i in 1:as.numeric(nsam-1)){kk$Cp[i]<-1-(kk$ue[i+1]/(kk$ue[1]))} 
@@ -388,26 +338,177 @@ deplet<-function(catch=NULL,effort=NULL,method=c("l","d","ml","hosc","hesc","hem
        kk$Akb<-cumsum(kk$effort)/kk$effort
         kk$I<-ifelse(kk$Cp>0,1,0)
        for(i in 1:nsam){
-        if(i==1){ kk$sql[i]<-0;kk$sql2[i]<-0}
-        if(i>1){
-           kk$sql[i]<-(kk$effort[i]^2/kk$catch[i])*(kk$Dk[i-1]+(kk$Ak[i-1]*kk$catch[i-1]*epsilonb))*kk$I[i-1]
-           kk$sql2[i]<-(kk$effort[i]^2/kk$catch[i])*kk$Cp[i-1]*kk$I[i-1]
+         if(kk$catch[i]>0){
+           if(i==1){ kk$sql[i]<-0;kk$sql2[i]<-0}
+           if(i>1){
+             kk$sql[i]<-(kk$effort[i]^2/kk$catch[i])*(kk$Dk[i-1]+(kk$Ak[i-1]*kk$catch[i-1]*epsilonb))*kk$I[i-1]
+             kk$sql2[i]<-(kk$effort[i]^2/kk$catch[i])*kk$Cp[i-1]*kk$I[i-1]
+           }
          }
-       }
+        if(kk$catch[i]==0){kk$sql[i]<-0;kk$sql2[i]<-0}
+      }
        Nmqb<-sum(kk$sql)/sum(kk$sql2)
-       BootNmq[j,1]<-i
+       qmqb<-kk$catch[1]/(Nmqb*kk$effort[1])
+       BootNmq[j,1]<-j
        BootNmq[j,2]<-Nmqb
+       BootNmq[j,3]<-qmqb
     }
          seN<-sd(BootNmq[,2])
+         seqq<-sd(BootNmq[,3])
          C<-exp(1.96*sqrt(log(1+((seN^2)/((Nmqle-sum(dd$catch))^2)))))  
          LCn<-sum(dd$catch)+(Nmqle-sum(dd$catch))/C
          UCn<-sum(dd$catch)+(Nmqle-sum(dd$catch))*C
          ans<-NULL  
-         ans$results<-matrix(NA,1L,7L)
-         ans$results<-rbind(cbind(round(Nmqle,0),round(seN,1),round(LCn,1),round(UCn,1),round(CVNmqle,3)))   
-         dimnames(ans$results)<-list(cbind("N"),c("Estimate","SE","95% LCI","95% UCI"," lambda CV"))  
+         ans$results<-matrix(NA,2L,7L)
+         ans$results<-rbind(cbind(round(Nmqle,0),round(seN,1),round(LCn,1),round(UCn,1),round(CVNmqle,3)),
+				    cbind(q,seqq,NA,NA,NA))   
+         dimnames(ans$results)<-list(cbind("N","lambda"),c("Estimate","SE","95% LCI","95% UCI"," lambda CV"))  
          hemqle.out<<-ans
+        
+  }
+if(any(method=="wh")){
+    wh.out<-NULL
+    nsam<-length(catch)
+    endk<-ifelse(is.null(kwh),as.numeric(nsam-2),kwh)
+    endk<-ifelse(endk>as.numeric(nsam-2),as.numeric(nsam-2),endk)
+    if(nsam==2){
+        warning ("Two pass estimator and variance of Otis et al. 1978 used.")
+        N2<-catch[1]/(1-(catch[2]/catch[1]))
+        q1<-1-(catch[2]/catch[1])
+        SEO<-sqrt(((N2*(1-q1)^nsam)*(1-(1-q1)^nsam))/((1-(1-q1)^nsam)^2-(nsam^2*q1^2*(1-q1)^(nsam-1))))
+        ans<-NULL 
+  	     ans$two_pass<-matrix(NA,2L,2L)
+           ans$two_pass<-rbind(cbind(round(N2,1),round(SEO,2)),
+                              cbind(q1,NA))
+        dimnames(ans$two_pass)<-list(cbind("N","q1"),c("Estimate"," Otis SE"))
+      }
+   if(nsam>2){
+		pcatch1<-c(rep(0,nsam))
+		T<-c(rep(0,nsam))
+      	qq<-seq(1,nsam,1)
+		T[c(seq(1,nsam,1))]<-cumsum(catch[c(seq(1,nsam,1))])
+		TC<-sum(catch)
+		N<-TC
+      ans<-NULL
+      resids<-NULL
+      preds<-NULL
+      lls<-NULL
+      for(k in 1:endk){# limited to n-p 
+            	if(k==1) ps<-T[nsam]/(nsam*N-sum(T[1:nsam-1]))  
+            	if(k>=2) ps<-rep(T[nsam]/(nsam*N-sum(T[1:nsam-1])),k)
+			parms<-c(N,ps)              
+      		model<-function(y){					
+   			  N<-y[1]
+  			  ps<-y[2:length(y)] 
+                    pp<-NULL 
+                    pp<-array(NA,dim=c(nsam,k))
+				for(nc in 1:ncol(pp)){
+	  			  for(nr in nc:nsam){
+					if(k==nc) pp[nr,nc]<-ps[nc]*(1-ps[nc])^(nr-nc)
+				      if(nc<k){
+					   if(nr==nc) pp[nr,nc]<-ps[nc]
+					   if(nr>nc) pp[nr,nc]<-1-ps[nc]
+             			}	
+          			   }
+       			}
+		pps<-apply(pp,1,prod,na.rm=T)
+            pcatch1<-N*pps
+ 		pT<-sum(pcatch1)
+  		L1<-sum(log((seq(1,T[nsam],1)+N-T[nsam])/seq(1,T[nsam],1)))
+             ratio<-NULL
+             for(f in 1:nsam){
+                if(catch[f]<=0 & pcatch1[f]>0) ratio[f]<-0
+                if(catch[f]<=0 & pcatch1[f]<=0) ratio[f]<-0
+                if(catch[f]>0 & pcatch1[f]<=0) ratio[f]<-0
+                if(catch[f]>0 & pcatch1[f]>0) ratio[f]<-log(catch[f]/pcatch1[f])
+               }
+
+          	H<-sum(catch*ratio)
+            dNTP<-ifelse(as.numeric(N-pT)<=0,0,log(N-pT))     
+  		G<-N*log(N)-T[nsam]*log(T[nsam])-(N-T[nsam])*dNTP-L1
+		G+H
+	   }   
+           	 lower<-c(sum(catch),rep(1e-15,k))                  
+		 upper<-c(sum(catch)*20,rep(1,k))                 #upper limits of N and q
+      		 j<-0
+      		 repeat{
+          	  		results<-try(optim(parms, model, gr = NULL,lower=lower,upper=upper,method=c("L-BFGS-B"), 
+          			control=list(maxit=100000),hessian=TRUE),TRUE)   
+                     if(class(results)=="try-error"){
+                		 j<-j+1
+                		 parms<-c(parms[1]*(1+j/10),parms[2:length(parms)])
+                		 if(j==30) break
+              	    }
+             	   if(class(results)!="try-error") break
+             	 }
+
+        if(class(results)!="try-error"){
+            cov1<-solve(results$hessian)
+            if(any(diag(cov1)<0)) cov1<-array(NA,dim=c(length(results$par),
+                       length(results$par)))
+                SEs<-sqrt(diag(cov1))
+                ps<-results$par[2:length(results$par)]
+                pp<-NULL
+                    pp<-array(NA,dim=c(nsam,length(ps)))
+				for(nc in 1:ncol(pp)){
+	  			  for(nr in nc:nsam){
+					if(k==nc) pp[nr,nc]<-ps[nc]*(1-ps[nc])^(nr-nc)
+				      if(nc<k){
+					   if(nr==nc) pp[nr,nc]<-ps[nc]
+					   if(nr>nc) pp[nr,nc]<-1-ps[nc]
+             			}	
+          			   }
+       			}
+			pps<-apply(pp,1,prod,na.rm=T)
+            	pcatch1<-results$par[1]*pps
+           		resid<-catch-pcatch1
+            	chi<-sum(((catch-pcatch1)^2)/pcatch1)
+            	df<-nsam-length(results$par)
+            	prob<-1-pchisq(chi,df)
+                  aic<-2*results$value+2*length(results$par)
+          }
+           if(class(results)=="try-error"){
+                  pcatch1<-rep(NA,1:nsam)
+           		resid<-rep(NA,1:nsam)
+            	chi<-NA
+            	df<-NA
+            	prob<-NA
+			SEs<-rep(NA,length(parms))
+           }
+       # Create CAPTURE TABLE  
+                   plabs<-paste("p",seq(1,length(ps),1),sep="")
+                   comb<-cbind(round(results$par,4),round(SEs,4))
+                   	rownames(comb)<-c("N",plabs)
+                   	comb<-as.data.frame(comb)
+                  	names(comb)<-c("Estimate","SE")
+                    comb$chi<-c(round(chi,3),rep(NA,length(ps))) 
+                    comb$df<- c(df,rep(NA,length(ps))) 
+			  comb$prob<-c(round(prob,4),rep(NA,length(ps))) 
+        # Create Residuals, Observed, Predicted Table
+             resids<-cbind(resids,resid)
+             preds<-cbind(preds,pcatch1)
+             lls<-rbind(lls,aic)
+          	 ans[[k]]<-comb  
+      }#Kloop
+           
+            prc<-as.matrix(cbind(catch,preds))
+             klabs<-paste("k=",seq(1,endk,1),sep="")         
+              colnames(prc)<-c("catch",klabs)
+            res<-as.matrix(resids)
+               colnames(res)<-c(klabs)
+             kl<-paste("k",seq(1,endk,1),sep="")
+            names(ans)<-c(kl)
+              aicout<-as.matrix(as.vector(lls),rownames=NA)
+                colnames(aicout)<-c("AIC Value")
+                rownames(aicout)<-c(seq(1,endk,1))
+            ans$aic<-aicout
+            ans$predicted<-prc
+		ans$residuals<-res
+                  
+        }   
+         wh.out<<-ans
   }
- }
 }
 
+
+
